@@ -23,22 +23,16 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <taglib.h>
-#include <tdebug.h>
-#include "trefcounter.h"
 #include "mp4item.h"
 
 using namespace TagLib;
 
-class MP4::Item::ItemPrivate : public RefCounter
+class MP4::Item::ItemPrivate
 {
 public:
-  ItemPrivate() :
-    valid(true),
-    atomDataType(TypeUndefined) {}
-
-  bool valid;
-  AtomDataType atomDataType;
+  Type type;
+  bool valid { true };
+  AtomDataType atomDataType { TypeUndefined };
   union {
     bool m_bool;
     int m_int;
@@ -53,90 +47,86 @@ public:
 };
 
 MP4::Item::Item() :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::Void;
   d->valid = false;
 }
 
-MP4::Item::Item(const Item &item) :
-  d(item.d)
-{
-  d->ref();
-}
-
-MP4::Item &
-MP4::Item::operator=(const Item &item)
-{
-  Item(item).swap(*this);
-  return *this;
-}
+MP4::Item::Item(const Item &) = default;
+MP4::Item &MP4::Item::operator=(const Item &) = default;
 
 void
-MP4::Item::swap(Item &item)
+MP4::Item::swap(Item &item) noexcept
 {
   using std::swap;
 
   swap(d, item.d);
 }
 
-MP4::Item::~Item()
-{
-  if(d->deref())
-    delete d;
-}
+MP4::Item::~Item() = default;
 
 MP4::Item::Item(bool value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::Bool;
   d->m_bool = value;
 }
 
 MP4::Item::Item(int value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::Int;
   d->m_int = value;
 }
 
 MP4::Item::Item(unsigned char value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::Byte;
   d->m_byte = value;
 }
 
 MP4::Item::Item(unsigned int value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::UInt;
   d->m_uint = value;
 }
 
 MP4::Item::Item(long long value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::LongLong;
   d->m_longlong = value;
 }
 
 MP4::Item::Item(int value1, int value2) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::IntPair;
   d->m_intPair.first = value1;
   d->m_intPair.second = value2;
 }
 
 MP4::Item::Item(const ByteVectorList &value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::ByteVectorList;
   d->m_byteVectorList = value;
 }
 
 MP4::Item::Item(const StringList &value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::StringList;
   d->m_stringList = value;
 }
 
 MP4::Item::Item(const MP4::CoverArtList &value) :
-  d(new ItemPrivate())
+  d(std::make_shared<ItemPrivate>())
 {
+  d->type = Type::CoverArtList;
   d->m_coverArtList = value;
 }
 
@@ -209,3 +199,47 @@ MP4::Item::isValid() const
 {
   return d->valid;
 }
+
+MP4::Item::Type MP4::Item::type() const
+{
+  return d->type;
+}
+
+bool MP4::Item::operator==(const Item &other) const
+{
+  if(isValid() && other.isValid() &&
+     type() == other.type() &&
+     atomDataType() == other.atomDataType()) {
+    switch(type()) {
+    case Type::Void:
+      return true;
+    case Type::Bool:
+      return toBool() == other.toBool();
+    case Type::Int:
+      return toInt() == other.toInt();
+    case Type::IntPair: {
+      const auto lhs = toIntPair();
+      const auto rhs = other.toIntPair();
+      return lhs.first == rhs.first && lhs.second == rhs.second;
+    }
+    case Type::Byte:
+      return toByte() == other.toByte();
+    case Type::UInt:
+      return toUInt() == other.toUInt();
+    case Type::LongLong:
+      return toLongLong() == other.toLongLong();
+    case Type::StringList:
+      return toStringList() == other.toStringList();
+    case Type::ByteVectorList:
+      return toByteVectorList() == other.toByteVectorList();
+    case Type::CoverArtList:
+      return toCoverArtList() == other.toCoverArtList();
+    }
+  }
+  return false;
+}
+
+bool MP4::Item::operator!=(const Item &other) const
+{
+  return !(*this == other);
+}
