@@ -27,31 +27,20 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tstring.h>
-#include <tdebug.h>
-#include <tpropertymap.h>
-#include <tagutils.h>
-
 #include "speexfile.h"
 
+#include "tdebug.h"
+#include "tpropertymap.h"
+#include "tagutils.h"
+
 using namespace TagLib;
 using namespace TagLib::Ogg;
 
 class Speex::File::FilePrivate
 {
 public:
-  FilePrivate() :
-    comment(0),
-    properties(0) {}
-
-  ~FilePrivate()
-  {
-    delete comment;
-    delete properties;
-  }
-
-  Ogg::XiphComment *comment;
-  Properties *properties;
+  std::unique_ptr<Ogg::XiphComment> comment;
+  std::unique_ptr<Properties> properties;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -63,7 +52,7 @@ bool Ogg::Speex::File::isSupported(IOStream *stream)
   // A Speex file has IDs "OggS" and "Speex   " somewhere.
 
   const ByteVector buffer = Utils::readHeader(stream, bufferSize(), false);
-  return (buffer.find("OggS") >= 0 && buffer.find("Speex   ") >= 0);
+  return buffer.find("OggS") >= 0 && buffer.find("Speex   ") >= 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -72,7 +61,7 @@ bool Ogg::Speex::File::isSupported(IOStream *stream)
 
 Speex::File::File(FileName file, bool readProperties, Properties::ReadStyle) :
   Ogg::File(file),
-  d(new FilePrivate())
+  d(std::make_unique<FilePrivate>())
 {
   if(isOpen())
     read(readProperties);
@@ -80,20 +69,17 @@ Speex::File::File(FileName file, bool readProperties, Properties::ReadStyle) :
 
 Speex::File::File(IOStream *stream, bool readProperties, Properties::ReadStyle) :
   Ogg::File(stream),
-  d(new FilePrivate())
+  d(std::make_unique<FilePrivate>())
 {
   if(isOpen())
     read(readProperties);
 }
 
-Speex::File::~File()
-{
-  delete d;
-}
+Speex::File::~File() = default;
 
 Ogg::XiphComment *Speex::File::tag() const
 {
-  return d->comment;
+  return d->comment.get();
 }
 
 PropertyMap Speex::File::properties() const
@@ -108,13 +94,13 @@ PropertyMap Speex::File::setProperties(const PropertyMap &properties)
 
 Speex::Properties *Speex::File::audioProperties() const
 {
-  return d->properties;
+  return d->properties.get();
 }
 
 bool Speex::File::save()
 {
   if(!d->comment)
-    d->comment = new Ogg::XiphComment();
+    d->comment = std::make_unique<Ogg::XiphComment>();
 
   setPacket(1, d->comment->render());
 
@@ -137,8 +123,8 @@ void Speex::File::read(bool readProperties)
 
   ByteVector commentHeaderData = packet(1);
 
-  d->comment = new Ogg::XiphComment(commentHeaderData);
+  d->comment = std::make_unique<Ogg::XiphComment>(commentHeaderData);
 
   if(readProperties)
-    d->properties = new Properties(this);
+    d->properties = std::make_unique<Properties>(this);
 }
