@@ -23,10 +23,12 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tdebug.h>
-#include <tfile.h>
-
 #include "infotag.h"
+
+#include <utility>
+
+#include "tbytevector.h"
+#include "tpropertymap.h"
 #include "riffutils.h"
 
 using namespace TagLib;
@@ -36,7 +38,7 @@ namespace
 {
   const RIFF::Info::StringHandler defaultStringHandler;
   const RIFF::Info::StringHandler *stringHandler = &defaultStringHandler;
-}
+} // namespace
 
 class RIFF::Info::Tag::TagPrivate
 {
@@ -44,17 +46,17 @@ public:
   FieldListMap fieldListMap;
 };
 
+class RIFF::Info::StringHandler::StringHandlerPrivate
+{
+};
+
 ////////////////////////////////////////////////////////////////////////////////
 // StringHandler implementation
 ////////////////////////////////////////////////////////////////////////////////
 
-StringHandler::StringHandler()
-{
-}
+StringHandler::StringHandler() = default;
 
-StringHandler::~StringHandler()
-{
-}
+StringHandler::~StringHandler() = default;
 
 String RIFF::Info::StringHandler::parse(const ByteVector &data) const
 {
@@ -71,20 +73,17 @@ ByteVector RIFF::Info::StringHandler::render(const String &s) const
 ////////////////////////////////////////////////////////////////////////////////
 
 RIFF::Info::Tag::Tag(const ByteVector &data) :
-  d(new TagPrivate())
+  d(std::make_unique<TagPrivate>())
 {
   parse(data);
 }
 
 RIFF::Info::Tag::Tag() :
-  d(new TagPrivate())
+  d(std::make_unique<TagPrivate>())
 {
 }
 
-RIFF::Info::Tag::~Tag()
-{
-  delete d;
-}
+RIFF::Info::Tag::~Tag() = default;
 
 String RIFF::Info::Tag::title() const
 {
@@ -167,6 +166,85 @@ bool RIFF::Info::Tag::isEmpty() const
   return d->fieldListMap.isEmpty();
 }
 
+namespace
+{
+  const Map<ByteVector, String> propertyKeyForId = {
+    {"IPRD", "ALBUM"},
+    {"IENG", "ARRANGER"},
+    {"IART", "ARTIST"},
+    {"IBSU", "ARTISTWEBPAGE"},
+    {"IBPM", "BPM"},
+    {"ICMT", "COMMENT"},
+    {"IMUS", "COMPOSER"},
+    {"ICOP", "COPYRIGHT"},
+    {"ICRD", "DATE"},
+    {"PRT1", "DISCSUBTITLE"},
+    {"ITCH", "ENCODEDBY"},
+    {"ISFT", "ENCODING"},
+    {"IDIT", "ENCODINGTIME"},
+    {"IGNR", "GENRE"},
+    {"ISRC", "ISRC"},
+    {"IPUB", "LABEL"},
+    {"ILNG", "LANGUAGE"},
+    {"IWRI", "LYRICIST"},
+    {"IMED", "MEDIA"},
+    {"ISTR", "PERFORMER"},
+    {"ICNT", "RELEASECOUNTRY"},
+    {"IEDT", "REMIXER"},
+    {"INAM", "TITLE"},
+    {"IPRT", "TRACKNUMBER"}
+  };
+}  // namespace
+
+PropertyMap RIFF::Info::Tag::properties() const
+{
+  PropertyMap props;
+  for(const auto &[id, val] : std::as_const(d->fieldListMap)) {
+    if(String key = propertyKeyForId.value(id); !key.isEmpty()) {
+      props[key].append(val);
+    }
+    else {
+      props.addUnsupportedData(key);
+    }
+  }
+  return props;
+}
+
+void RIFF::Info::Tag::removeUnsupportedProperties(const StringList &props)
+{
+  for(const auto &id : props)
+    d->fieldListMap.erase(id.data(String::Latin1));
+}
+
+PropertyMap RIFF::Info::Tag::setProperties(const PropertyMap &props)
+{
+  static Map<String, ByteVector> idForPropertyKey;
+  if(idForPropertyKey.isEmpty()) {
+    for(const auto &[id, key] : propertyKeyForId) {
+      idForPropertyKey[key] = id;
+    }
+  }
+
+  const PropertyMap origProps = properties();
+  for(const auto &[key, _] : origProps) {
+    if(!props.contains(key) || props.value(key).isEmpty()) {
+      d->fieldListMap.erase(idForPropertyKey.value(key));
+    }
+  }
+
+  PropertyMap ignoredProps;
+  for(const auto &[key, val] : props) {
+    if(ByteVector id = idForPropertyKey.value(key);
+       !id.isEmpty() && !val.isEmpty()) {
+      d->fieldListMap[id] = val.front();
+    }
+    else {
+      ignoredProps.insert(key, val);
+    }
+  }
+  return ignoredProps;
+}
+
 FieldListMap RIFF::Info::Tag::fieldListMap() const
 {
   return d->fieldListMap;
@@ -181,7 +259,7 @@ String RIFF::Info::Tag::fieldText(const ByteVector &id) const
 
 void RIFF::Info::Tag::setFieldText(const ByteVector &id, const String &s)
 {
-  // id must be four-byte long pure ascii string.
+  // id must be a four-byte long pure ascii string.
   if(!isValidChunkName(id))
     return;
 
@@ -201,13 +279,12 @@ ByteVector RIFF::Info::Tag::render() const
 {
   ByteVector data("INFO");
 
-  FieldListMap::ConstIterator it = d->fieldListMap.begin();
-  for(; it != d->fieldListMap.end(); ++it) {
-    ByteVector text = stringHandler->render(it->second);
+  for(const auto &[field, list] : std::as_const(d->fieldListMap)) {
+    ByteVector text = stringHandler->render(list);
     if(text.isEmpty())
       continue;
 
-    data.append(it->first);
+    data.append(field);
     data.append(ByteVector::fromUInt(text.size() + 1, false));
     data.append(text);
 
@@ -241,8 +318,7 @@ void RIFF::Info::Tag::parse(const ByteVector &data)
     if(size > data.size() - p - 8)
       break;
 
-    const ByteVector id = data.mid(p, 4);
-    if(isValidChunkName(id)) {
+    if(const ByteVector id = data.mid(p, 4); isValidChunkName(id)) {
       const String text = stringHandler->parse(data.mid(p + 8, size));
       d->fieldListMap[id] = text;
     }
