@@ -27,56 +27,44 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tstring.h>
-#include <tdebug.h>
-#include <bitset>
-
 #include "trueaudioproperties.h"
-#include "trueaudiofile.h"
+
+#include "tdebug.h"
+#include "tstring.h"
 
 using namespace TagLib;
 
 class TrueAudio::Properties::PropertiesPrivate
 {
 public:
-  PropertiesPrivate(const ByteVector &d, long length, ReadStyle s) :
-    data(d),
-    streamLength(length),
-    style(s),
-    version(0),
-    length(0),
-    bitrate(0),
-    sampleRate(0),
-    channels(0),
-    bitsPerSample(0) {}
-
-  ByteVector data;
-  long streamLength;
-  ReadStyle style;
-  int version;
-  int length;
-  int bitrate;
-  int sampleRate;
-  int channels;
-  int bitsPerSample;
+  int version { 0 };
+  int length { 0 };
+  int bitrate { 0 };
+  int sampleRate { 0 };
+  int channels { 0 };
+  int bitsPerSample { 0 };
+  unsigned int sampleFrames { 0 };
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-TrueAudio::Properties::Properties(const ByteVector &data, long streamLength, ReadStyle style) : AudioProperties(style)
+TrueAudio::Properties::Properties(const ByteVector &data, offset_t streamLength, ReadStyle style) :
+  AudioProperties(style),
+  d(std::make_unique<PropertiesPrivate>())
 {
-  d = new PropertiesPrivate(data, streamLength, style);
-  read();
+  read(data, streamLength);
 }
 
-TrueAudio::Properties::~Properties()
+TrueAudio::Properties::~Properties() = default;
+
+int TrueAudio::Properties::lengthInSeconds() const
 {
-  delete d;
+  return d->length / 1000;
 }
 
-int TrueAudio::Properties::length() const
+int TrueAudio::Properties::lengthInMilliseconds() const
 {
   return d->length;
 }
@@ -101,6 +89,11 @@ int TrueAudio::Properties::channels() const
   return d->channels;
 }
 
+unsigned int TrueAudio::Properties::sampleFrames() const
+{
+  return d->sampleFrames;
+}
+
 int TrueAudio::Properties::ttaVersion() const
 {
   return d->version;
@@ -110,34 +103,49 @@ int TrueAudio::Properties::ttaVersion() const
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-void TrueAudio::Properties::read()
+void TrueAudio::Properties::read(const ByteVector &data, offset_t streamLength)
 {
-  if(!d->data.startsWith("TTA"))
+  if(data.size() < 4) {
+    debug("TrueAudio::Properties::read() -- data is too short.");
     return;
+  }
 
-  int pos = 3;
+  if(!data.startsWith("TTA")) {
+    debug("TrueAudio::Properties::read() -- invalid header signature.");
+    return;
+  }
 
-  d->version = d->data[pos] - '0';
+  unsigned int pos = 3;
+
+  d->version = data[pos] - '0';
   pos += 1;
 
   // According to http://en.true-audio.com/TTA_Lossless_Audio_Codec_-_Format_Description
   // TTA2 headers are in development, and have a different format
   if(1 == d->version) {
+    if(data.size() < 18) {
+      debug("TrueAudio::Properties::read() -- data is too short.");
+      return;
+    }
+
     // Skip the audio format
     pos += 2;
 
-    d->channels = d->data.mid(pos, 2).toShort(false);
+    d->channels = data.toShort(pos, false);
     pos += 2;
 
-    d->bitsPerSample = d->data.mid(pos, 2).toShort(false);
+    d->bitsPerSample = data.toShort(pos, false);
     pos += 2;
 
-    d->sampleRate = d->data.mid(pos, 4).toUInt(false);
+    d->sampleRate = data.toUInt(pos, false);
     pos += 4;
 
-    uint sampleFrames = d->data.mid(pos, 4).toUInt(false);
-    d->length = d->sampleRate > 0 ? sampleFrames / d->sampleRate : 0;
+    d->sampleFrames = data.toUInt(pos, false);
 
-    d->bitrate = d->length > 0 ? ((d->streamLength * 8L) / d->length) / 1000 : 0;
+    if(d->sampleFrames > 0 && d->sampleRate > 0) {
+      const auto length = static_cast<double>(d->sampleFrames) * 1000.0 / d->sampleRate;
+      d->length  = static_cast<int>(length + 0.5);
+      d->bitrate = static_cast<int>(streamLength * 8.0 / length + 0.5);
+    }
   }
 }
