@@ -24,14 +24,13 @@
  ***************************************************************************/
 
 #include <string>
-#include <stdio.h>
-#include <tag.h>
-#include <tstringlist.h>
-#include <tbytevectorlist.h>
-#include <tpropertymap.h>
-#include <apefile.h>
-#include <apetag.h>
-#include <tdebug.h>
+#include <cstdio>
+
+#include "tstringlist.h"
+#include "tpropertymap.h"
+#include "tag.h"
+#include "apefile.h"
+#include "apetag.h"
 #include <cppunit/extensions/HelperMacros.h>
 #include "utils.h"
 
@@ -79,7 +78,7 @@ public:
     tag.setProperties(dict);
     CPPUNIT_ASSERT_EQUAL(String("17"), tag.itemListMap()["TRACK"].values()[0]);
     CPPUNIT_ASSERT_EQUAL(2u, tag.itemListMap()["ARTIST"].values().size());
-    CPPUNIT_ASSERT_EQUAL(String("artist 1 artist 2"), tag.artist());
+    CPPUNIT_ASSERT_EQUAL(String("artist 1 / artist 2"), tag.artist());
     CPPUNIT_ASSERT_EQUAL(17u, tag.track());
     const APE::Item &textItem = tag.itemListMap()["TRACK"];
     CPPUNIT_ASSERT_EQUAL(APE::Item::Text, textItem.type());
@@ -90,10 +89,10 @@ public:
   void testPropertyInterface2()
   {
     APE::Tag tag;
-    APE::Item item1 = APE::Item("TRACK", "17");
+    APE::Item item1("TRACK", String("17"));
     tag.setItem("TRACK", item1);
 
-    APE::Item item2 = APE::Item();
+    APE::Item item2;
     item2.setType(APE::Item::Binary);
     ByteVector binaryData1("first");
     item2.setBinaryData(binaryData1);
@@ -118,10 +117,10 @@ public:
     tag.removeUnsupportedProperties(properties.unsupportedData());
     CPPUNIT_ASSERT(!tag.itemListMap().contains("TESTBINARY"));
 
-    APE::Item item3 = APE::Item("TRACKNUMBER", "29");
+    APE::Item item3("TRACKNUMBER", String("29"));
     tag.setItem("TRACKNUMBER", item3);
     properties = tag.properties();
-    CPPUNIT_ASSERT_EQUAL((unsigned int)2, properties["TRACKNUMBER"].size());
+    CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(2), properties["TRACKNUMBER"].size());
     CPPUNIT_ASSERT_EQUAL(String("17"), properties["TRACKNUMBER"][0]);
     CPPUNIT_ASSERT_EQUAL(String("29"), properties["TRACKNUMBER"][1]);
 
@@ -138,21 +137,21 @@ public:
 
     APE::Tag tag;
     PropertyMap unsuccessful = tag.setProperties(properties);
-    CPPUNIT_ASSERT_EQUAL((unsigned int)3, unsuccessful.size());
+    CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(3), unsuccessful.size());
     CPPUNIT_ASSERT(unsuccessful.contains("A"));
     CPPUNIT_ASSERT(unsuccessful.contains("MP+"));
     CPPUNIT_ASSERT(unsuccessful.contains(L"\x1234\x3456"));
 
-    CPPUNIT_ASSERT_EQUAL((unsigned int)2, tag.itemListMap().size());
+    CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(2), tag.itemListMap().size());
     tag.addValue("VALID KEY", "Test Value 1");
     tag.addValue("INVALID KEY \x7f", "Test Value 2");
     tag.addValue(L"INVALID KEY \x1234\x3456", "Test Value 3");
-    CPPUNIT_ASSERT_EQUAL((unsigned int)3, tag.itemListMap().size());
+    CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(3), tag.itemListMap().size());
   }
 
   void testTextBinary()
   {
-    APE::Item item = APE::Item("DUMMY", "Test Text");
+    APE::Item item("DUMMY", String("Test Text"));
     CPPUNIT_ASSERT_EQUAL(String("Test Text"), item.toString());
     CPPUNIT_ASSERT_EQUAL(ByteVector(), item.binaryData());
 
@@ -188,4 +187,3 @@ public:
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestAPETag);
-
